# See morph/grid/local/grid.nix for additional commentary.
let
  pkgs = import <nixpkgs> { };

  gridlib = import ../../lib;
  rawConfig = pkgs.lib.trivial.importJSON ./config.json;
  config = rawConfig // {
    sshUsers = import ./public-keys/users.nix;

    publicKeyPath = toString ./. + "/${rawConfig.publicKeyPath}";
    privateKeyPath = toString ./. + "/${rawConfig.privateKeyPath}";
  };

  # Configure deployment management authorization for all systems in the grid.
  grid-config = {
    imports = [
      gridlib.base
    ];

    services.private-storage.deployment = {
      authorizedKey = builtins.readFile "${config.publicKeyPath}/deploy_key.pub";
      gridName = "production";
    };
    services.private-storage.sshUsers = config.sshUsers;
    grid = {
      inherit (config) publicKeyPath privateKeyPath; 
    };
  };

  payments = {
    imports = [
      gridlib.issuer
      gridlib.hardware-aws
      (gridlib.customize-issuer (config // {
        monitoringvpnIPv4 = "172.23.23.11";
      }))
      grid-config
    ];
    monitoringvpnIPv4 = "172.23.23.11";
  };

  monitoring = {
    imports = [
      gridlib.monitoring
      gridlib.hardware-aws
      (gridlib.customize-monitoring {
        inherit paymentExporterTargets;
        inherit (config) domain letsEncryptAdminEmail;
        googleOAuthClientID = config.monitoringGoogleOAuthClientID;
        monitoringvpnIPv4 = "172.23.23.1";
      })
      grid-config
    ];
    monitoringvpnIPv4 = "172.23.23.1";
    system.stateVersion = "19.09";
  };

  defineStorageNode = name: { vpnIP, stateVersion }:
  let
    nodecfg = import "${./.}/${name}-config.nix";
    hardware ="${./.}/${name}-hardware.nix";
  in {
    imports = [
      # Get some of the very lowest-level system configuration for this
      # node.  This isn't all *completely* hardware related.  Maybe some
      # more factoring is in order, someday.
      hardware

      # Slightly awkwardly, enable some of our hardware / network / bootloader options.
      ../../../nixos/modules/100tb.nix

      # Get all of the configuration that is common across all storage nodes.
      gridlib.storage

      # Then customize the storage system a little bit based on this node's particulars.
      (gridlib.customize-storage (config // nodecfg // {
        monitoringvpnIPv4 = vpnIP;
      }))

      # Also configure deployment management authorization
      grid-config
    ];

    monitoringvpnIPv4 = vpnIP;
    system.stateVersion = stateVersion;
    # And supply configuration for those hardware / network / bootloader
    # options.  See the 100tb module for handling of this value.  The module
    # name is quoted because `1` makes `100tb` look an awful lot like a
    # number.
   "100tb".config = nodecfg;
  };

  # Define all of the storage nodes for this grid.
  storageNodes = builtins.mapAttrs defineStorageNode {
    storage001 = { vpnIP = "172.23.23.21"; stateVersion = "19.09"; };
    storage002 = { vpnIP = "172.23.23.22"; stateVersion = "19.09"; };
    storage003 = { vpnIP = "172.23.23.23"; stateVersion = "19.09"; };
    storage004 = { vpnIP = "172.23.23.24"; stateVersion = "19.09"; };
    storage005 = { vpnIP = "172.23.23.25"; stateVersion = "19.03"; };
  };

  # TBD: derive these automatically:
  paymentExporterTargets = [ "payments" ];

in {
  network = {
    description = "PrivateStorage.io Production Grid";
  };
  inherit payments;
  inherit monitoring;
} // storageNodes
